#include "adc12.h"
#include "p30f6011.h"
#include "can.h"
#include "i2c.h"
#include "display_leds.h"
#include "pin_definitions.h"

void write_freq(void);
void config_VCO(void);
void write_freq_running(void);
//unsigned long frequency_Hz;
extern union mixed_char_int {
    int i[4];
    unsigned char c[8];
} CAN_Buffer;
extern int PSU_mode;

struct freq_tag {
    unsigned int freq_adc;
    char freq_offset;
    unsigned int devider;
    float freq;
    unsigned change:1;
} frequency;
//I2C communication varaibles
unsigned int config2, config1;
//frequency setup
union hl_full {
    unsigned int full;
    struct {
        unsigned char l;
        unsigned char h;
    } HL;
};

union hl_full freq;
unsigned int freq_offset_base;
unsigned char freq_offset;
unsigned int pause;

int I2C_Loop_Counter;
char first_run;
extern unsigned int pause;
extern long err_int_low;
extern union {
    long full;
    int top[2];
} err_int;

static union hl_full fword;
extern char operating_mode;

void config_VCO(void)
{
//configure frequency synthesiser
    config2 = 50;                                          //160;
    config1 = (I2C_ON & I2C_IDLE_CON & I2C_CLK_HLD
               & I2C_IPMI_DIS & I2C_7BIT_ADD
               & I2C_SLW_DIS & I2C_SM_DIS
               & I2C_GCALL_DIS & I2C_STR_DIS
               & I2C_NACK & I2C_ACK_DIS & I2C_RCV_DIS
               & I2C_STOP_DIS & I2C_RESTART_DIS & I2C_START_DIS);
    OpenI2C(config1, config2);
    IdleI2C();

    for (pause = 0; pause < 1000; pause++);

//this part sets up the address of the frequency synthesiser
// and prevents the (oscillator) eeprom from being written to

    StartI2C();
    while (I2CCONbits.SEN);
    MasterWriteI2C(0xb0);
    while (I2CSTATbits.TRSTAT);
    while (I2CSTATbits.ACKSTAT);
    MasterWriteI2C(0x0D);
    while (I2CSTATbits.TRSTAT);
    while (I2CSTATbits.ACKSTAT);
    MasterWriteI2C(0x08);
    while (I2CSTATbits.TRSTAT);
    while (I2CSTATbits.ACKSTAT);
    StopI2C();
    while (I2CCONbits.PEN);
    IdleI2C();

//wait for the synthesiser to recover from that signal

    for (pause = 0; pause < 32000; pause++);

// find out the offset value of the frequency synthesiser
// send comand
    StartI2C();
    while (I2CCONbits.SEN);
    MasterWriteI2C(0xb0);
    while (I2CSTATbits.TRSTAT);
    while (I2CSTATbits.ACKSTAT);
    MasterWriteI2C(0x37);
    while (I2CSTATbits.TRSTAT);
    while (I2CSTATbits.ACKSTAT);
// read data
    RestartI2C();
    while (I2CCONbits.RSEN);
    MasterWriteI2C(0xb1);
    while (I2CSTATbits.TRSTAT);
    while (I2CSTATbits.ACKSTAT);
    freq.HL.h = MasterReadI2C();
    AckI2C;
    while (I2CSTATbits.ACKSTAT);
    freq.HL.l = MasterReadI2C();
    NotAckI2C;
    while (I2CSTATbits.ACKSTAT);
    StopI2C();
    while (I2CCONbits.PEN);
    IdleI2C();

    for (pause = 0; pause < 1000; pause++);

    freq_offset_base = (freq.full & 0xf800) >> 11;

    freq_offset = freq_offset_base;

    StartI2C();
    while (I2CCONbits.SEN);
    MasterWriteI2C(0xb0);
    while (I2CSTATbits.TRSTAT);
    while (I2CSTATbits.ACKSTAT);
    MasterWriteI2C(0x0e);
    while (I2CSTATbits.TRSTAT);
    while (I2CSTATbits.ACKSTAT);
    MasterWriteI2C(freq_offset);
    while (I2CSTATbits.TRSTAT);
    while (I2CSTATbits.ACKSTAT);
    StopI2C();
    while (I2CCONbits.PEN);
    IdleI2C();

    for (pause = 0; pause < 1000; pause++);
//send dividing outputs to div1
    StartI2C();
    while (I2CCONbits.SEN);
    MasterWriteI2C(0xb0);
    while (I2CSTATbits.TRSTAT);
    while (I2CSTATbits.ACKSTAT);
    MasterWriteI2C(0x01);
    while (I2CSTATbits.TRSTAT);
    while (I2CSTATbits.ACKSTAT);
    MasterWriteI2C(0x00);
    while (I2CSTATbits.TRSTAT);
    while (I2CSTATbits.ACKSTAT);
    MasterWriteI2C(0x80);
    while (I2CSTATbits.TRSTAT);
    while (I2CSTATbits.ACKSTAT);
    StopI2C();
    while (I2CCONbits.PEN);
    IdleI2C();

//wait for the synthesiser to recover from that signal

    for (pause = 0; pause < 1000; pause++);
}
void write_freq(void)
{
    union hl_full divword;

    if (I2C_Loop_Counter == 0) {
        //errlights.errbits.COMMS_FAULT=0;
    } else {
        errlights.errbits.COMMS_FAULT = 1;
        return;
    }

    frequency.change = 0;

    while (frequency.freq_adc > 43000) {
        frequency.freq_offset++;
        frequency.freq_adc -= 16384;
        frequency.change = 1;
    }

    while (frequency.freq_adc < 22000) {
        frequency.freq_offset--;
        frequency.freq_adc += 16384;
        frequency.change = 1;
    }

    while (frequency.freq_offset > 6) {
        frequency.devider = frequency.devider / 2;
        frequency.freq_offset -= 10;
        frequency.change = 1;
    }

    while (frequency.freq_offset < (-6)) {
        frequency.devider = frequency.devider * 2;
        frequency.freq_offset += 10;
        frequency.change = 1;
    }




    frequency.freq =
        (((float) frequency.freq_adc) * 0.762939 +
         ((float) frequency.freq_offset) * 12500.0 +
         175000.0) / ((float) (frequency.devider));

    divword.full = (frequency.devider - 2) << 6;

//master_slave=!frequency.change;



    if ((frequency.change == 1) || (first_run == 1)) {


        IEC0bits.MI2CIE = 0;                               //disable the I2C interrupt - we are doing this bit manually
        freq_offset = freq_offset_base + frequency.freq_offset;

        StartI2C();
        while (I2CCONbits.SEN);
        MasterWriteI2C(0xb0);                              //address
        while (I2CSTATbits.TRSTAT);
        while (I2CSTATbits.ACKSTAT);
        MasterWriteI2C(0x0e);                              //offset command
        while (I2CSTATbits.TRSTAT);
        while (I2CSTATbits.ACKSTAT);
        MasterWriteI2C(freq_offset);                       //data
        while (I2CSTATbits.TRSTAT);
        while (I2CSTATbits.ACKSTAT);
        StopI2C();
        while (I2CCONbits.PEN);
        IdleI2C();
        for (pause = 0; pause < 1000; pause++);

        StartI2C();                                        //send devider info
        while (I2CCONbits.SEN);
        MasterWriteI2C(0xb0);                              //address
        while (I2CSTATbits.TRSTAT);
        while (I2CSTATbits.ACKSTAT);
        MasterWriteI2C(0x01);                              //devider command
        while (I2CSTATbits.TRSTAT);
        while (I2CSTATbits.ACKSTAT);
        MasterWriteI2C(divword.HL.h);                      //msb
        while (I2CSTATbits.TRSTAT);
        while (I2CSTATbits.ACKSTAT);
        MasterWriteI2C(divword.HL.l);                      //lsb
        while (I2CSTATbits.TRSTAT);
        while (I2CSTATbits.ACKSTAT);
        StopI2C();
        while (I2CCONbits.PEN);
        IdleI2C();

        for (pause = 0; pause < 1000; pause++);

    }



    fword.full = frequency.freq_adc;

    IEC0bits.MI2CIE = 0;
    StartI2C();
    while (I2CCONbits.SEN);
    I2C_Loop_Counter = 2;
    IFS0bits.MI2CIF = 0;
    MasterWriteI2C(0xb0);
    IEC0bits.MI2CIE = 1;
}

void __attribute__ ((__interrupt__)) _MI2CInterrupt(void)
{
    IFS0bits.MI2CIF = 0;

    switch (I2C_Loop_Counter++) {
    case 0:
        {
            I2C_Loop_Counter = 0;
            break;
        }
    case 1:
        {
            I2C_Loop_Counter = 0;
            break;
            while (I2CCONbits.SEN);
            MasterWriteI2C(0xb0);
            break;
        }
    case 2:
        {
            while (I2CSTATbits.TRSTAT);
            while (I2CSTATbits.ACKSTAT);
            MasterWriteI2C(0x08);
            break;
        }
    case 3:
        {
            while (I2CSTATbits.TRSTAT);
            while (I2CSTATbits.ACKSTAT);
            MasterWriteI2C(fword.HL.h);
            break;
        }
    case 4:
        {
            while (I2CSTATbits.TRSTAT);
            while (I2CSTATbits.ACKSTAT);
            MasterWriteI2C(fword.HL.l);
            break;
        }
    case 5:
        {
            while (I2CSTATbits.TRSTAT);
            while (I2CSTATbits.ACKSTAT);
            StopI2C();
            break;
        }
    case 6:
        {
            while (I2CCONbits.PEN);
            IdleI2C;
            I2C_Loop_Counter = 0;
            break;
        }
    default:
        {
            I2C_Loop_Counter = 0;
        }
    }
}
